// Importing the required modules from D3
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the number on the x-axis
const xAxisTickFormat = number =>
    format('.3s')(number)

//Rendering the data
const render = data => { 
// Assigning X axis label (Year)   
  const xValue = d => d.Year; 
  const xAxisLabel = 'Year';
// Assigning Y axis label (Annual Income) 
  const yValue = d => d.Annual_NetIncome_In_Millions;
  const yAxisLabel = 'Annual_NetIncome_In_Millions';
 
//Adding margins
  const margin = { top: 10, right: 40, bottom: 88, left: 120 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top-30 - margin.bottom-20;
  
  // Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue)) 
    .range([0, innerWidth]) 
    .nice(); //round values out
  
// Scale for Y axis  
  const yScale = scaleLinear()
    .domain([0, max(data, yValue )])
    .range([innerHeight, 0])
    .nice();

// Positioning the svg element  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
 
// Assigning values to x axis  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(20);
 
// Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
  ;
  	
//Appending text to the x and y axis  
  const yAxisG = g.append('g').call(yAxis);
  	yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -90)
      .attr('x', -innerHeight / 6)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 70)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);

//Generating line to svg 
  
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)));
  
  g.append('path')
  	.attr('class', 'line_path')
  	.attr('d', lineGenerator(data))
};

// Loading and Parsing CSV data
csv('https://gist.githubusercontent.com/nagasai-gummadi/6d6c370a943222612293338107d2e7c3/raw/24e70ff59bb4dd7c198d17b0dd42d6ee4c2f3744/income.csv')
  .then(data => {
    data.forEach(d => {
    	d.Annual_NetIncome_In_Millions = +d.Annual_NetIncome_In_Millions;
//Formatting to retrieve year from the date
      d.Year = d3.timeParse("%Y-%m-%d")(d.Year);
    });

    render(data);
  });